from ..lib.eobot_config import EobotConfig, get_config
from ..lib.eobot_errors import NoEmailError, NoPasswordOrTokenError
from ..lib.eobot_request import EobotRequest


def perform_request(config=None, request=None):
    """
    Retrieves the user id for the current user

    :param config : (Optional) Configuration to use, will default to the global config if not provided
    :type config : EobotConfig|str|None

    :param request : (Optional) Request object to use, will default to a new one if not provided
    :type request : EobotRequest|None

    :rtype : int
    """
    if request is None:
        request = EobotRequest()
    elif not isinstance(request, EobotRequest):
        raise ValueError("Invalid request, must be a EobotRequet")

    if config is None or isinstance(config, str):
        config = get_config(config)
    elif not isinstance(config, EobotConfig):
        raise ValueError("Invalid config, must be a EobotConfig")

    if not config.has_email():
        raise NoEmailError()

    if not config.has_token() and not config.has_password():
        raise NoPasswordOrTokenError()

    request.set_parameter("email", config.get_email())
    request.set_parameter("password", config.get_token() if config.has_token() else config.get_password())

    result = request.perform_request()

    return int(result["userid"])



from .eobot_errors import NoUserIdError, NoEmailError, NoPasswordOrTokenError
from .eobot_authentication import EobotWriteAuthentication, EobotReadonlyAuthentication

# used as default values in EobotConfig.configure() because None is actually a valid value and will drop current values,
# which is unwanted if named arguments are not provided
_NONE = '__none__'


class EobotConfig(object):
    """
    The EobotConfig object is used to provide user credentials to API calls, where needed.
    """

    def __init__(self):
        super(EobotConfig, self).__init__()

        self._user_id = None
        self._email = None
        self._password = None
        self._token = None

    def configure(self, user_id=_NONE, email=_NONE, password=_NONE, token=_NONE):
        """
        Configures the user credentials

        :param user_id  : (Optional) Eobot user ID to use for methods that only require that
        :param email    : (Optional) Eobot username to use for methods that perform changes
        :param password : (Optional) Eobot password to use for methods that perform changes, if the user has the API
                          configured to use password instead of tokens
        :param token    : (Optional) Eobot API token to use for methods that perform changes, if the user has the API
                          configured to use tokens instead of password

        :type user_id  : int|None
        :type email    : str|None
        :type password : str|None
        :type token    : str|None

        :returns EobotConfig : the current instance, for easy method chaining
        :rtype : EobotConfig
        """
        if user_id != _NONE:
            self.set_user_id(user_id)

        if email != _NONE:
            self.set_email(email)

        if password != _NONE:
            self.set_password(password)

        if token != _NONE:
            self.set_token(token)

        return self

    def has_user_id(self):
        """
        Returns whether a user_id is configured

        :rtype : bool
        """
        return self._user_id is not None

    def set_user_id(self, user_id):
        """
        Sets the user id

        :param user_id : user_id to configure, must be an int, can be None to clear any previous user_id
        :type user_id : int|None
        :raises ValueError : if the user_id is not an int

        :returns EobotConfig : the current instance, for easy method chaining
        :rtype : EobotConfig
        """
        if user_id is not None and not isinstance(user_id, int):
            raise ValueError("Invalid user_id, it must be an int")

        self._user_id = user_id
        return self

    def get_user_id(self):
        """
        Returns the currently configured user_id

        :rtype : int
        """
        return self._user_id

    def has_email(self):
        """
        Returns whether an email is configured

        :rtype : bool
        """
        return self._email is not None

    def set_email(self, email):
        """
        Sets the email

        :param email : email to configure, must be a str, can be None to clear any previous email
        :type email : str|None
        :raises ValueError : if the email is not a str

        :returns EobotConfig : the current instance, for easy method chaining
        :rtype : EobotConfig
        """
        if email is not None and not isinstance(email, str):
            raise ValueError("Invalid email, it must be a str")

        self._email = email
        return self

    def get_email(self):
        """
        Returns the currently configured email

        :rtype : str
        """
        return self._email

    def has_password(self):
        """
        Returns whether a password is configured

        :rtype : bool
        """
        return self._password is not None

    def set_password(self, password):
        """
        Sets the password

        :param password : password to configure, must be a str, can be None to clear any previous password
        :type password : str|None
        :raises ValueError : if the password is not a str

        :returns EobotConfig : the current instance, for easy method chaining
        :rtype : EobotConfig
        """
        if password is not None and not isinstance(password, str):
            raise ValueError("Invalid password, it must be a str")

        self._password = password
        return self

    def get_password(self):
        """
        Returns the currently configured password

        :rtype : str
        """
        return self._password

    def has_token(self):
        """
        Returns whether a token is configured

        :rtype : bool
        """
        return self._token is not None

    def set_token(self, token):
        """
        Sets the token

        :param token : token to configure, must be a str, can be None to clear any previous token
        :type token : str|None
        :raises ValueError : if the token is not a str

        :returns EobotConfig : the current instance, for easy method chaining
        :rtype : EobotConfig
        """
        if token is not None and not isinstance(token, str):
            raise ValueError("Invalid token, it must be a str")

        self._token = token
        return self

    def get_token(self):
        """
        Returns the currently configured token

        :rtype : str
        """
        return self._token

    def get_authentication(self, readonly=True):
        """
        Returns the authentication parameters needed for an API call. Readonly methods only need a user_id, write-calls
        require an email and token/password

        :param readonly : (Optional) Whether to return authentication parameters for a readonly API call or for a
                          write-call
        :type readonly  : bool
        :rtype          : EobotReadonlyAuthentication|EobotWriteAuthentication
        """
        if not self.has_user_id():
            raise NoUserIdError()

        if not readonly and not self.has_email():
            raise NoEmailError()

        if not readonly and not self.has_token() and not self.has_password():
            raise NoPasswordOrTokenError()

        if readonly:
            return EobotReadonlyAuthentication(self.get_user_id())

        if self.has_token():
            return EobotWriteAuthentication(self.get_user_id(), self.get_email(), self.get_token())

        return EobotWriteAuthentication(self.get_user_id(), self.get_email(), self.get_password())


_configs = {
    "__global": EobotConfig()
}


def get_config(name=None):
    """
    Returns an `EobotConfig` object that is used to set up user credentials for use in all API calls

    :param name : (Optional) Can be used to retrieve a particular configuration. If not provided, the global config is
                  returned
    :type name  : str
    :rtype      : EobotConfig
    """
    if name is None:
        name = "__global"

    if not isinstance(name, str):
        raise ValueError("Invalid config name, it must be a str")

    if name not in _configs.keys():
        _configs[name] = EobotConfig()

    return _configs[name]



from ..lib.eobot_config import EobotConfig
from ..lib.eobot_request import EobotRequest


# noinspection PyUnusedLocal
def perform_request(config=None, request=None):
    """
    Retrieves the current values in US dollar for all supported cryptocurrencies. 1 coin equals {result} US dollar.

    :param config : Not used for this request, since this API method does not require authentication
    :type config : EobotConfig|str|None

    :param request : (Optional) Request object to use, will default to a new one if not provided
    :type request : EobotRequest|None

    :rtype : dict
    """
    if request is None:
        request = EobotRequest()
    elif not isinstance(request, EobotRequest):
        raise ValueError("Invalid request, must be a EobotRequet")

    request.set_parameter("supportedcoins", "true")
    request.set_parameter("currency", "USD")

    result = request.perform_request()

    for coin in result.keys():
        result[coin]["Price"] = float(result[coin]["Price"])

    return result



from .get_balances import perform_request as get_balances
from .get_coin_value import perform_request as get_coin_value
from .get_deposit_address import perform_request as get_deposit_address
from .get_exchange_estimate import perform_request as get_exchange_estimate
from .get_exchange_rate import perform_request as get_exchange_rate
from .get_mining_mode import perform_request as get_mining_mode
from .get_mining_speed import perform_request as get_mining_speed
from .get_supported_coins import perform_request as get_supported_coins
from .get_supported_fiat import perform_request as get_supported_fiat
from .get_user_id import perform_request as get_user_id
from .manual_withdraw import perform_request as manual_withdraw
from .set_automatic_withdraw import perform_request as set_automatic_withdraw
from .set_mining_mode import perform_request as set_mining_mode



class EobotReadonlyAuthentication(object):
    """
    Contains the Eobot user_id for use when authenticating against the API
    """
    def __init__(self, user_id):
        """
        :param user_id : Eobot user_id to set
        :type user_id  : int
        """
        super(EobotReadonlyAuthentication, self).__init__()
        if not isinstance(user_id, int):
            raise ValueError("Invalid user_id, must be an int")
        self.user_id = user_id


class EobotWriteAuthentication(EobotReadonlyAuthentication):
    """
    Contains the Eobot email address and password/token for use when authenticating against the API, optionally also
    includes the user_id
    """
    def __init__(self, user_id, email, password_or_token):
        """
        :param user_id           : Eobot user_id to set
        :param email             : Email address to set
        :param password_or_token : Password or token to set

        :type user_id           : int
        :type email             : str
        :type password_or_token : str
        """
        super(EobotWriteAuthentication, self).__init__(user_id)
        if not isinstance(email, str):
            raise ValueError("Invalid email, must be a str")
        self.email = email
        if not isinstance(password_or_token, str):
            raise ValueError("Invalid password_or_token, must be a str")
        self.password = password_or_token



from ..lib.eobot_config import EobotConfig
from ..lib.eobot_request import EobotRequest


# noinspection PyUnusedLocal
def perform_request(coin, config=None, request=None):
    """
    Retrieves the current value for `coin` in US dollar. 1 `coin` equals {result} US dollar

    :param coin : Cryptocurrency to retrieve value for
    :type coin : str

    :param config : Not used for this request, since this API method does not require authentication
    :type config : EobotConfig|str|None

    :param request : (Optional) Request object to use, will default to a new one if not provided
    :type request : EobotRequest|None

    :rtype : str
    """
    if request is None:
        request = EobotRequest()
    elif not isinstance(request, EobotRequest):
        raise ValueError("Invalid request, must be a EobotRequet")

    if not isinstance(coin, str):
        raise ValueError("Invalid coin, must be a str")

    coin = coin.upper()

    request.set_parameter("coin", coin)

    result = request.perform_request()

    return float(result[coin])



from ..lib.eobot_config import EobotConfig, get_config
from ..lib.eobot_errors import NoUserIdError
from ..lib.eobot_request import EobotRequest
from .get_balances import perform_request as get_balances
from .get_user_id import perform_request as get_user_id


def perform_request(from_coin, amount, to_coin, config=None, request=None):
    """
    Exchanges `amount` `from_coin` to `to_coin` (note: Eobot will withhold a percentage as fee when doing this)

    :param from_coin : Cryptocurrency to exchange from
    :type from_coin : str

    :param amount : Amount to exchange
    :type amount : int|float

    :param to_coin : Cryptocurrency to exchange to
    :type to_coin : str

    :param config : (Optional) Configuration to use, will default to the global config if not provided
    :type config : EobotConfig|str|None

    :param request : (Optional) Request object to use, will default to a new one if not provided
    :type request : EobotRequest|None

    :rtype : bool
    """
    if not isinstance(from_coin, str):
        raise ValueError("Invalid from_coin, must be a str")

    from_coin = from_coin.upper()

    if not isinstance(amount, float) and not isinstance(amount, int):
        raise ValueError("Invalid amount, must be a float or int")

    if not isinstance(to_coin, str):
        raise ValueError("Invalid to_coin, must be a str")

    to_coin = to_coin.upper()

    if request is None:
        request = EobotRequest()
    elif not isinstance(request, EobotRequest):
        raise ValueError("Invalid request, must be a EobotRequet")

    if config is None or isinstance(config, str):
        config = get_config(config)
    elif not isinstance(config, EobotConfig):
        raise ValueError("Invalid config, must be a EobotConfig")

    try:
        config.get_authentication(True)
    except NoUserIdError:
        config.set_user_id(get_user_id(config=config, request=request.clone()))

    auth = config.get_authentication(False)

    old_balances = get_balances(config=config, request=request.clone())
    old_balance_from = old_balances[from_coin] if from_coin in old_balances.keys() else 0.0
    old_balance_to = old_balances[to_coin] if to_coin in old_balances.keys() else 0.0

    request.set_parameter("id", auth.user_id)
    request.set_parameter("email", auth.email)
    request.set_parameter("password", auth.password)
    request.set_parameter("convertfrom", from_coin)
    request.set_parameter("amount", amount)
    request.set_parameter("convertto", to_coin)
    request.perform_request()

    new_balances = get_balances(config=config, request=request.clone())
    new_balance_from = new_balances[from_coin] if from_coin in new_balances.keys() else 0.0
    new_balance_to = new_balances[to_coin] if to_coin in new_balances.keys() else 0.0

    return new_balance_from < old_balance_from and new_balance_to > old_balance_to



class NoUserIdError(RuntimeError):
    """
    Raised when an API method is called that requires a user id, and no user id is set in the current config
    """
    pass


class NoEmailError(RuntimeError):
    """
    Raised when an API method is called that requires a user email address, and no email is set in the current config
    """
    pass


class NoPasswordOrTokenError(RuntimeError):
    """
    Raised when an API method is called that requires either an API token or user password, and neither is set in the
    current config
    """
    pass



from ..lib.eobot_config import EobotConfig, get_config
from ..lib.eobot_errors import NoUserIdError
from ..lib.eobot_request import EobotRequest
from .get_user_id import perform_request as get_user_id


def perform_request(config=None, request=None):
    """
    Retrieves the current mining mode for the current user

    :param config : (Optional) Configuration to use, will default to the global config if not provided
    :type config : EobotConfig|str|None

    :param request : (Optional) Request object to use, will default to a new one if not provided
    :type request : EobotRequest|None

    :rtype : str
    """
    if request is None:
        request = EobotRequest()
    elif not isinstance(request, EobotRequest):
        raise ValueError("Invalid request, must be a EobotRequet")

    if config is None or isinstance(config, str):
        config = get_config(config)
    elif not isinstance(config, EobotConfig):
        raise ValueError("Invalid config, must be a EobotConfig")

    try:
        auth = config.get_authentication(True)
    except NoUserIdError:
        config.set_user_id(get_user_id(config=config, request=request.clone()))
        auth = config.get_authentication(True)

    request.set_parameter("idmining", auth.user_id)

    result = request.perform_request()

    return result["mining"]



from ..lib.eobot_config import EobotConfig
from ..lib.eobot_request import EobotRequest


# noinspection PyUnusedLocal
def perform_request(from_coin, to_coin, amount, config=None, request=None):
    """
    Retrieves the estimated amount of `to_coin` you'd get for `amount` `from_coin`

    :param from_coin : Cryptocurrency to exchange from
    :type from_coin : str

    :param to_coin : Cryptocurrency to exchange to
    :type to_coin : str

    :param amount : Amount to exchange
    :type amount : float|int

    :param config : Not used for this request, since this API method does not require authentication
    :type config : EobotConfig|str|None

    :param request : (Optional) Request object to use, will default to a new one if not provided
    :type request : EobotRequest|None

    :rtype : float
    """
    if request is None:
        request = EobotRequest()
    elif not isinstance(request, EobotRequest):
        raise ValueError("Invalid request, must be a EobotRequet")

    if not isinstance(from_coin, str):
        raise ValueError("Invalid from_coin, must be a str")

    if not isinstance(to_coin, str):
        raise ValueError("Invalid to_coin, must be a str")

    if not isinstance(amount, int) and not isinstance(amount, float):
        raise ValueError("Invalid amount, must be a float or int")

    from_coin = from_coin.upper()
    to_coin = to_coin.upper()

    request.set_parameter("exchangefee", "true")
    request.set_parameter("convertfrom", from_coin)
    request.set_parameter("amount", amount)
    request.set_parameter("convertto", to_coin)

    result = request.perform_request()

    return float(result["Result"])



from ..lib.eobot_config import EobotConfig, get_config
from ..lib.eobot_errors import NoUserIdError
from ..lib.eobot_request import EobotRequest
from .get_user_id import perform_request as get_user_id
from .get_mining_mode import perform_request as get_mining_mode


def perform_request(mode, config=None, request=None):
    """
    Changes the mining mode to the cryptocurrency specified

    :param mode : Mining mode to set
    :type mode : str

    :param config : (Optional) Configuration to use, will default to the global config if not provided
    :type config : EobotConfig|str|None

    :param request : (Optional) Request object to use, will default to a new one if not provided
    :type request : EobotRequest|None

    :rtype : bool
    """
    if not isinstance(mode, str):
        raise ValueError("Invalid mode, must be a str")

    mode = mode.upper()

    if request is None:
        request = EobotRequest()
    elif not isinstance(request, EobotRequest):
        raise ValueError("Invalid request, must be a EobotRequet")

    if config is None or isinstance(config, str):
        config = get_config(config)
    elif not isinstance(config, EobotConfig):
        raise ValueError("Invalid config, must be a EobotConfig")

    try:
        config.get_authentication(True)
    except NoUserIdError:
        config.set_user_id(get_user_id(config=config, request=request.clone()))

    auth = config.get_authentication(False)

    current_mode = get_mining_mode(config=config, request=request.clone())
    if current_mode == mode:
        return True

    request.set_parameter("id", auth.user_id)
    request.set_parameter("email", auth.email)
    request.set_parameter("password", auth.password)
    request.set_parameter("mining", mode)
    request.perform_request()

    new_mode = get_mining_mode(config=config, request=request.clone())
    return new_mode == mode



from ..lib.eobot_config import EobotConfig, get_config
from ..lib.eobot_errors import NoUserIdError
from ..lib.eobot_request import EobotRequest
from .get_user_id import perform_request as get_user_id


def perform_request(coin, on_amount, wallet_address, config=None, request=None):
    """
    Sets the withdraw settings for `coin` to automatically withdraw to `wallet_address` when `on_amount` is reached

    :param coin : Cryptocurrency to configure
    :type coin : str

    :param on_amount : Threshold to reach for automatic withdrawal
    :type on_amount : int|float

    :param wallet_address : Wallet address to withdraw funds to
    :type wallet_address : str

    :param config : (Optional) Configuration to use, will default to the global config if not provided
    :type config : EobotConfig|str|None

    :param request : (Optional) Request object to use, will default to a new one if not provided
    :type request : EobotRequest|None

    :rtype : bool
    """
    if not isinstance(coin, str):
        raise ValueError("Invalid coin, must be a str")

    coin = coin.upper()

    if not isinstance(on_amount, float) and not isinstance(on_amount, int):
        raise ValueError("Invalid on_amount, must be a float or int")

    if not isinstance(wallet_address, str):
        raise ValueError("Invalid wallet_address, must be a str")

    if request is None:
        request = EobotRequest()
    elif not isinstance(request, EobotRequest):
        raise ValueError("Invalid request, must be a EobotRequet")

    if config is None or isinstance(config, str):
        config = get_config(config)
    elif not isinstance(config, EobotConfig):
        raise ValueError("Invalid config, must be a EobotConfig")

    try:
        config.get_authentication(True)
    except NoUserIdError:
        config.set_user_id(get_user_id(config=config, request=request.clone()))

    auth = config.get_authentication(False)

    request.set_parameter("id", auth.user_id)
    request.set_parameter("email", auth.email)
    request.set_parameter("password", auth.password)
    request.set_parameter("withdraw", coin)
    request.set_parameter("amount", on_amount)
    request.set_parameter("wallet", wallet_address)
    request.perform_request()

    # This API method does not return anything by which we can see whether the request was successful, so assume it was
    return True



from setuptools import setup
import re


def readme():
    with open('README.rst') as f:
        return f.read()


VERSIONFILE = "eobot/_version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__\s+=\s+['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    verstr = 'unknown'

setup(
    name='eobot-py',
    version=verstr,
    description='Eobot API wrapper',
    long_description=readme(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: Freeware',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Utilities',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries'
    ],
    keywords='eobot eobot.com api',
    url='http://github.com/rickdenhaan/eobot-py',
    author='Rick den Haan',
    author_email='rick@capirussa.nl',
    license='Freeware',
    packages=['eobot'],
    include_package_data=True,
    install_requires=[
        'requests'
    ],
    zip_safe=False,
    test_suite='nose.collector',
    tests_require=['nose']
)



from .._version import __version__
import requests


class EobotRequest(object):
    """
    Wrapper around the `requests` library to perform API requests
    """
    def __init__(self):
        super(EobotRequest, self).__init__()

        self._timeout = 30.0
        self._validate_ssl = True
        self._user_agent = 'RickDenHaan-Eobot/{0} (+http://github.com/rickdenhaan/eobot-py)'.format(__version__)
        self._base_url = 'https://www.eobot.com/api.aspx'
        self._parameters = {}

    def set_timeout(self, timeout):
        """
        Sets the request timeout

        :param timeout : timeout in seconds
        :type timeout : float|int

        :returns EobotRequest : the current instance, for easy method chaining
        :rtype : EobotRequest
        """
        if not isinstance(timeout, float) and not isinstance(timeout, int):
            raise ValueError("Invalid timeout, must be a float or int")

        self._timeout = float(timeout)
        return self

    def get_timeout(self):
        """
        Returns the current request timeout

        :rtype : float
        """
        return self._timeout

    def set_validate_ssl(self, validate_ssl):
        """
        Sets whether or not to validate the remote server's SSL certificate

        :param validate_ssl : whether or not to validate the certificate
        :type validate_ssl : bool

        :returns EobotRequest : the current instance, for easy method chaining
        :rtype : EobotRequest
        """
        if not isinstance(validate_ssl, bool):
            raise ValueError("Invalid validate_ssl, must be a bool")

        self._validate_ssl = validate_ssl
        return self

    def get_validate_ssl(self):
        """
        Returns whether or not to validate the remote server's SSL certificate

        :rtype : bool
        """
        return self._validate_ssl

    def set_user_agent(self, user_agent):
        """
        Sets the user agent for the request

        :param user_agent : user agent for the request
        :type user_agent : str

        :returns EobotRequest : the current instance, for easy method chaining
        :rtype : EobotRequest
        """
        if not isinstance(user_agent, str):
            raise ValueError("Invalid user_agent, must be a str")

        self._user_agent = user_agent
        return self

    def get_user_agent(self):
        """
        Returns the current user agent

        :rtype : str
        """
        return self._user_agent

    def set_base_url(self, base_url):
        """
        Sets the base URL for the request

        :param base_url : base URL for the request
        :type base_url : str

        :returns EobotRequest : the current instance, for easy method chaining
        :rtype : EobotRequest
        """
        if not isinstance(base_url, str):
            raise ValueError("Invalid base_url, must be a str")

        self._base_url = base_url
        return self

    def get_base_url(self):
        """
        Returns the current base URL for the request

        :rtype : str
        """
        return self._base_url

    def set_parameters(self, parameters):
        """
        Sets the request parameters

        :param parameters : request parameters
        :type parameters : dict

        :returns EobotRequest : the current instance, for easy method chaining
        :rtype : EobotRequest
        """
        if not isinstance(parameters, dict):
            raise ValueError("Invalid parameters, must be a dict")

        self._parameters = parameters
        return self

    def set_parameter(self, parameter, value):
        """
        Sets a specific request parameter

        :param parameter : parameter to set
        :param value : value to set for the parameter

        :type parameter : str
        :type value : str|int|float|list|dict|tuple

        :returns EobotRequest : the current instance, for easy method chaining
        :rtype : EobotRequest
        """
        if not isinstance(parameter, str):
            raise ValueError("Invalid parameter, must be a str")

        self._parameters[parameter] = value
        return self

    def get_parameters(self):
        """
        Returns the current request parameters

        :rtype : dict
        """
        return self._parameters

    def clone(self):
        """
        Creates a new request object with the same properties as this one

        :rtype : EobotRequest
        """
        clone = EobotRequest()
        clone.set_timeout(self.get_timeout())
        clone.set_validate_ssl(self.get_validate_ssl())
        clone.set_user_agent(self.get_user_agent())
        clone.set_base_url(self.get_base_url())

        return clone

    def perform_request(self):
        """
        Performs the API request and returns the response value

        :rtype : dict
        """
        url = self.get_base_url()
        headers = {
            "User-Agent": self.get_user_agent()
        }

        parameters = self.get_parameters()
        parameters["json"] = "true"

        response = requests.get(
            url,
            params=parameters,
            headers=headers,
            timeout=self.get_timeout(),
            verify=self.get_validate_ssl()
        )

        try:
            return response.json()
        except:
            raise RuntimeError("Unexpected non-JSON response: {0}".format(response.text))
